@@ -1,4 +1,11 @@
+#pragma GCC diagnostic ignored "-Wunused-function"
+
 #include "homcloud_common.h"
+#include "homcloud_cgal.h"
+#include "homcloud_numpy.h"
+
+#ifdef CGAL_NEWER_API_4_11
+
 #include "alpha_shape_common.h"
 
 #include <CGAL/Exact_predicates_inexact_constructions_kernel.h>
@@ -10,7 +17,6 @@
 #include <CGAL/Alpha_shape_3.h>
 #include <CGAL/Alpha_shape_cell_base_3.h>
 #include <CGAL/Alpha_shape_vertex_base_3.h>
-#include <CGAL/Robust_weighted_circumcenter_filtered_traits_3.h>
 
 #include <map>
 #include <vector>
@@ -176,10 +182,17 @@ static double AlphaValueEdge(CGAL_AlphaShape3* alpha_shape,E e)
 /////////////// AlphaShape ///////////////////////////
 static int PeriodicAlphaShape3_init(PeriodicAlphaShape3* self, PyObject* args, PyObject* kwds) {
   PyArrayObject* points;
-  PyObject* bc;
   int weighted, rel_homology;
+  double xmin;
+  double xmax;
+  double ymin;
+  double ymax;
+  double zmin;
+  double zmax;
 
-  if (!PyArg_ParseTuple(args, "O!ppO", &PyArray_Type, &points, &weighted, &rel_homology, &bc)) 
+  if (!PyArg_ParseTuple(args, "O!ppdddddd", &PyArray_Type, &points,
+                        &weighted, &rel_homology, 
+                        &xmin, &xmax, &ymin, &ymax, &zmin, &zmax))
     return -1;
 
   if (!ArrayHasCorrectShape(points, 3, weighted, rel_homology))
@@ -187,44 +200,7 @@ static int PeriodicAlphaShape3_init(PeriodicAlphaShape3* self, PyObject* args, P
   if (!ArrayIsDoubleType(points))
     return -1;
 
-  double xmin = 0.0;
-  double xmax = 1.0;
-  double ymin = 0.0;
-  double ymax = 1.0;
-  double zmin = 0.0;
-  double zmax = 1.0;
-
-  PyObject* arg = Py_BuildValue("()");
-  PyObject* xrange = PyObject_GetAttrString(bc,"xrange");
-  PyObject* yrange = PyObject_GetAttrString(bc,"yrange");
-  PyObject* zrange = PyObject_GetAttrString(bc,"zrange");
-  if( xrange != nullptr && PyCallable_Check(xrange) ){
-    PyObject* result = PyObject_CallObject(xrange, arg);
-    xmin = PyFloat_AsDouble(PyTuple_GetItem(result,0));
-    xmax = PyFloat_AsDouble(PyTuple_GetItem(result,1));
-    Py_XDECREF(xrange);
-    Py_XDECREF(result);
-  }
-
-  if( yrange != nullptr && PyCallable_Check(yrange) ){
-    PyObject* result = PyObject_CallObject(yrange, arg);
-    ymin = PyFloat_AsDouble(PyTuple_GetItem(result,0));
-    ymax = PyFloat_AsDouble(PyTuple_GetItem(result,1));
-    Py_XDECREF(yrange);
-    Py_XDECREF(result);
-  }
-
-  if( zrange != nullptr && PyCallable_Check(zrange) ){
-    PyObject* result = PyObject_CallObject(zrange, arg);
-    zmin = PyFloat_AsDouble(PyTuple_GetItem(result,0));
-    zmax = PyFloat_AsDouble(PyTuple_GetItem(result,1));
-    Py_XDECREF(zrange);
-    Py_XDECREF(result);
-  }
-
-  Py_XDECREF(bc);
-
-  if (xmax-xmin != ymax-ymin || xmax-xmin != zmax-zmin) {
+  if (xmax - xmin != ymax - ymin || xmax - xmin != zmax - zmin) {
     PyErr_SetString(PyExc_ValueError, "Periodic condition must be cubical.");
     return -1;
   }
@@ -908,6 +884,7 @@ static PyMethodDef Vertex_methods[] = {
    "Return the coordinate of the vertex"},
   {"weight", (PyCFunction)Vertex_weight, METH_NOARGS,
    "Return the weight of the vertex"},
+  {"isvertex", (PyCFunction)method_return_true, METH_NOARGS, "Return True"},
   {NULL, NULL, 0 ,NULL}
 };
 
@@ -1017,6 +994,7 @@ static PyMethodDef Cell_methods[] = {
   {"vertices",
    (PyCFunction)Cell_vertices, METH_NOARGS,
    "Return vertices of the cell"},
+  {"isvertex", (PyCFunction)method_return_false, METH_NOARGS, "Return False"},
   {NULL, NULL, 0 ,NULL}
 };
 
@@ -1105,6 +1083,7 @@ error:
 static PyMethodDef Edge_methods[] = {
   {"vertices", (PyCFunction)Edge_vertices, METH_NOARGS,
    "Return endpoint vertices of the edge"},
+  {"isvertex", (PyCFunction)method_return_false, METH_NOARGS, "Return False"},
   {NULL, NULL, 0 ,NULL}
 };
 
@@ -1191,6 +1170,7 @@ error:
 static PyMethodDef Facet_methods[] = {
   {"vertices", (PyCFunction)Facet_vertices, METH_NOARGS,
    "Return endpoint vertices of the edge"},
+  {"isvertex", (PyCFunction)method_return_false, METH_NOARGS, "Return False"},
   {NULL, NULL, 0 ,NULL}
 };
 
@@ -1301,3 +1281,81 @@ extern "C" void initperiodic_alpha_shape3()
   return module;
 #endif
 }
+
+#else  // CGAL_NEWER_API_4_11
+
+typedef struct {
+  PyObject_HEAD
+} PeriodicAlphaShape3;
+
+static int PeriodicAlphaShape3_init(PeriodicAlphaShape3* self, PyObject* args, PyObject* kwds) {
+  PyErr_SetString(PyExc_RuntimeError, "Periodic Alpha filtration is not supported old CGAL");
+  return -1;
+}
+
+static PyTypeObject PeriodicAlphaShape3Type = {
+  PyVarObject_HEAD_INIT(NULL, 0)
+  "homcloud.periodic_alpha_shape3.PeriodicAlphaShape3", // tp_name
+  sizeof(PeriodicAlphaShape3), // tp_basicsize 
+  0, // tp_itemsize
+  0, // tp_dealloc
+  0, // tp_print
+  0, // tp_getattr
+  0, // tp_setattr
+  0, // tp_reserved
+  0, // tp_repr
+  0, // tp_as_number
+  0, // tp_as_sequence
+  0, // tp_as_mapping
+  0, // tp_hash 
+  0, // tp_call
+  0, // tp_str
+  0, // tp_getattro
+  0, // tp_setattro
+  0, // tp_as_buffer
+  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_BASETYPE, // tp_flags 
+  "Dummy Class for old CGAL (< 4.11)",  // tp_doc 
+  0, // tp_traverse
+  0, // tp_clear
+  0, // tp_richcompare
+  0, // tp_weaklistoffset
+  0, // tp_iter
+  0, // tp_iternext
+  0, // tp_methods 
+  0, // tp_members
+  0, // tp_getset
+  0, // tp_base
+  0, // tp_dict
+  0, // tp_descr_get
+  0, // tp_descr_set
+  0, // tp_dictoffset
+  reinterpret_cast<initproc>(PeriodicAlphaShape3_init), // tp_init 
+  0, // tp_alloc
+  PyType_GenericNew, // tp_new
+}; 
+
+static PyModuleDef periodic_alpha_shape3_Module = {
+  PyModuleDef_HEAD_INIT,
+  "homcloud.periodic_alpha_shape3",
+  "The module for 3D Periodic Alpha shape (dummy)",
+  -1,
+};
+
+PyMODINIT_FUNC
+PyInit_periodic_alpha_shape3()
+{
+  if (PyType_Ready(&PeriodicAlphaShape3Type) < 0)
+    return nullptr;
+
+  PyObject* module = PyModule_Create(&periodic_alpha_shape3_Module);
+
+  if (!module)
+    return nullptr;
+
+  Py_INCREF(&PeriodicAlphaShape3Type);
+  PyModule_AddObject(module, "PeriodicAlphaShape3", cast_PyObj(&PeriodicAlphaShape3Type));
+
+  return module;
+}
+
+#endif  // CGAL_NEWER_API_4_11
